#
# This files contains definitions needed to use CGAL in a program.
# DO NOT EDIT THIS. The definitons have been generated by CMake at configuration time.
# This file is loaded by cmake via the command "find_package(CGAL)"
#
# This file correspond to a possibly out-of-sources CGAL configuration, thus the actual location
# must be given by the cmake variable or enviroment variable CGAL_DIR.

set(CGAL_CONFIG_LOADED TRUE)

get_filename_component(CGAL_CONFIG_DIR "${CMAKE_CURRENT_LIST_FILE}" PATH)

set(CGAL_HEADER_ONLY "OFF" )

# The code for including exported targets is different from
# CGAL_Config_install.cmake. We do not have separate export files in
# an installed version and we need to make sure that we are not
# currently building CGAL. Note that for CMake<=2.8.11 (detected by
# the absence of CMP0024), the else() condition is never used.
if(NOT POLICY CMP0024 OR NOT CGAL_BUILDING_LIBS)
  include("${CGAL_CONFIG_DIR}/CGALExports.cmake")

  macro(CGAL_get_property_optional VAR TARGET PROPERTY)
    if(TARGET ${TARGET})
      get_property(${VAR} TARGET ${TARGET} PROPERTY ${PROPERTY})
    else()
      set(${VAR} "")
    endif()
  endmacro()

  if(NOT MSVC AND NOT CGAL_HEADER_ONLY)
    CGAL_get_property_optional(CGAL_LIBRARY CGAL::CGAL LOCATION)
    CGAL_get_property_optional(CGAL_Core_LIBRARY CGAL::CGAL_Core LOCATION)
    CGAL_get_property_optional(CGAL_ImageIO_LIBRARY CGAL::CGAL_ImageIO LOCATION)
    CGAL_get_property_optional(CGAL_Qt5_LIBRARY CGAL::CGAL_Qt5 LOCATION)
  else()
    set(CGAL_LIBRARY         "")
    set(CGAL_Core_LIBRARY    "")
    set(CGAL_ImageIO_LIBRARY "")
    set(CGAL_Qt5_LIBRARY     "")
  endif()
else()
  # We are currently in a CGAL Build and CGALExports.cmake has not
  # necessarily been created yet. Just alias the targets. Also don't
  # access the LOCATION property here to set lib_LIBRARY, since those
  # targets are not imported and this is disallowed by CMP0026. Just
  # set it to the target name.
  macro(CGAL_alias_library lib)
    if(TARGET ${lib} AND NOT TARGET CGAL::${lib})
      add_library(CGAL::${lib} ALIAS ${lib})
      set(${lib}_LIBRARY CGAL::${lib})
    else()
      set(${lib}_LIBRARY "")
    endif()
  endmacro()

  CGAL_alias_library(CGAL)
  CGAL_alias_library(CGAL_Core)
  CGAL_alias_library(CGAL_ImageIO)
  CGAL_alias_library(CGAL_Qt5)
endif()

# Check for possible config files of our libraries and include them.
file(GLOB CGAL_CONFIG_FILES LIST_DIRECTORIES false
  RELATIVE "${CGAL_CONFIG_DIR}" "${CGAL_CONFIG_DIR}/*LibConfig.cmake")
list(SORT CGAL_CONFIG_FILES)
foreach(configfile ${CGAL_CONFIG_FILES})
  include("${CGAL_CONFIG_DIR}/${configfile}")
endforeach()

set(CGAL_INSTALLATION_PACKAGE_DIR "/tmp/cgal-releases-CGAL-4.11/Installation")
set(CGAL_CORE_PACKAGE_DIR "/tmp/cgal-releases-CGAL-4.11/Core")
set(CGAL_GRAPHICSVIEW_PACKAGE_DIR "/tmp/cgal-releases-CGAL-4.11/GraphicsView")

set(CGAL_MAJOR_VERSION    "4" )
set(CGAL_MINOR_VERSION    "11" )
set(CGAL_BUGFIX_VERSION   "0" )
set(CGAL_BUILD_VERSION    "900" )
set(CGAL_SCM_BRANCH_NAME  "n/a")
set(CGAL_GIT_SHA1         "")

set(CGAL_BUILD_SHARED_LIBS        "ON" )
set(CGAL_Boost_USE_STATIC_LIBS    "OFF" )

set(CGAL_CXX_FLAGS_INIT                   " -frounding-math" )
set(CGAL_CXX_FLAGS_RELEASE_INIT           "-O3 -DNDEBUG" )
set(CGAL_CXX_FLAGS_DEBUG_INIT             "-g" )
set(CGAL_MODULE_LINKER_FLAGS_INIT         "" )
set(CGAL_MODULE_LINKER_FLAGS_RELEASE_INIT "" )
set(CGAL_MODULE_LINKER_FLAGS_DEBUG_INIT   "" )
set(CGAL_SHARED_LINKER_FLAGS_INIT         "" )
set(CGAL_SHARED_LINKER_FLAGS_RELEASE_INIT "" )
set(CGAL_SHARED_LINKER_FLAGS_DEBUG_INIT   "" )
set(CGAL_BUILD_TYPE_INIT                  "Release" )

set(CGAL_INCLUDE_DIRS      "/tmp/cgal-releases-CGAL-4.11/buildR/include;/tmp/cgal-releases-CGAL-4.11/AABB_tree/include;/tmp/cgal-releases-CGAL-4.11/Advancing_front_surface_reconstruction/include;/tmp/cgal-releases-CGAL-4.11/Algebraic_foundations/include;/tmp/cgal-releases-CGAL-4.11/Algebraic_kernel_d/include;/tmp/cgal-releases-CGAL-4.11/Algebraic_kernel_for_circles/include;/tmp/cgal-releases-CGAL-4.11/Algebraic_kernel_for_spheres/include;/tmp/cgal-releases-CGAL-4.11/Alpha_shapes_2/include;/tmp/cgal-releases-CGAL-4.11/Alpha_shapes_3/include;/tmp/cgal-releases-CGAL-4.11/Apollonius_graph_2/include;/tmp/cgal-releases-CGAL-4.11/Arithmetic_kernel/include;/tmp/cgal-releases-CGAL-4.11/Arrangement_on_surface_2/include;/tmp/cgal-releases-CGAL-4.11/BGL/include;/tmp/cgal-releases-CGAL-4.11/Barycentric_coordinates_2/include;/tmp/cgal-releases-CGAL-4.11/Boolean_set_operations_2/include;/tmp/cgal-releases-CGAL-4.11/Bounding_volumes/include;/tmp/cgal-releases-CGAL-4.11/Box_intersection_d/include;/tmp/cgal-releases-CGAL-4.11/CGAL_Core/include;/tmp/cgal-releases-CGAL-4.11/CGAL_ImageIO/include;/tmp/cgal-releases-CGAL-4.11/CGAL_ipelets/include;/tmp/cgal-releases-CGAL-4.11/Cartesian_kernel/include;/tmp/cgal-releases-CGAL-4.11/Circular_kernel_2/include;/tmp/cgal-releases-CGAL-4.11/Circular_kernel_3/include;/tmp/cgal-releases-CGAL-4.11/Circulator/include;/tmp/cgal-releases-CGAL-4.11/Combinatorial_map/include;/tmp/cgal-releases-CGAL-4.11/Cone_spanners_2/include;/tmp/cgal-releases-CGAL-4.11/Convex_decomposition_3/include;/tmp/cgal-releases-CGAL-4.11/Convex_hull_2/include;/tmp/cgal-releases-CGAL-4.11/Convex_hull_3/include;/tmp/cgal-releases-CGAL-4.11/Convex_hull_d/include;/tmp/cgal-releases-CGAL-4.11/Distance_2/include;/tmp/cgal-releases-CGAL-4.11/Distance_3/include;/tmp/cgal-releases-CGAL-4.11/Envelope_2/include;/tmp/cgal-releases-CGAL-4.11/Envelope_3/include;/tmp/cgal-releases-CGAL-4.11/Filtered_kernel/include;/tmp/cgal-releases-CGAL-4.11/Generalized_map/include;/tmp/cgal-releases-CGAL-4.11/Generator/include;/tmp/cgal-releases-CGAL-4.11/Geomview/include;/tmp/cgal-releases-CGAL-4.11/GraphicsView/include;/tmp/cgal-releases-CGAL-4.11/HalfedgeDS/include;/tmp/cgal-releases-CGAL-4.11/Hash_map/include;/tmp/cgal-releases-CGAL-4.11/Homogeneous_kernel/include;/tmp/cgal-releases-CGAL-4.11/Inscribed_areas/include;/tmp/cgal-releases-CGAL-4.11/Installation/include;/tmp/cgal-releases-CGAL-4.11/Interpolation/include;/tmp/cgal-releases-CGAL-4.11/Intersections_2/include;/tmp/cgal-releases-CGAL-4.11/Intersections_3/include;/tmp/cgal-releases-CGAL-4.11/Interval_skip_list/include;/tmp/cgal-releases-CGAL-4.11/Interval_support/include;/tmp/cgal-releases-CGAL-4.11/Inventor/include;/tmp/cgal-releases-CGAL-4.11/Jet_fitting_3/include;/tmp/cgal-releases-CGAL-4.11/Kernel_23/include;/tmp/cgal-releases-CGAL-4.11/Kernel_d/include;/tmp/cgal-releases-CGAL-4.11/Kinetic_data_structures/include;/tmp/cgal-releases-CGAL-4.11/Kinetic_framework/include;/tmp/cgal-releases-CGAL-4.11/LEDA/include;/tmp/cgal-releases-CGAL-4.11/Linear_cell_complex/include;/tmp/cgal-releases-CGAL-4.11/MacOSX/include;/tmp/cgal-releases-CGAL-4.11/Maintenance/include;/tmp/cgal-releases-CGAL-4.11/Matrix_search/include;/tmp/cgal-releases-CGAL-4.11/Mesh_2/include;/tmp/cgal-releases-CGAL-4.11/Mesh_3/include;/tmp/cgal-releases-CGAL-4.11/Mesher_level/include;/tmp/cgal-releases-CGAL-4.11/Minkowski_sum_2/include;/tmp/cgal-releases-CGAL-4.11/Minkowski_sum_3/include;/tmp/cgal-releases-CGAL-4.11/Modifier/include;/tmp/cgal-releases-CGAL-4.11/Modular_arithmetic/include;/tmp/cgal-releases-CGAL-4.11/Nef_2/include;/tmp/cgal-releases-CGAL-4.11/Nef_3/include;/tmp/cgal-releases-CGAL-4.11/Nef_S2/include;/tmp/cgal-releases-CGAL-4.11/NewKernel_d/include;/tmp/cgal-releases-CGAL-4.11/Number_types/include;/tmp/cgal-releases-CGAL-4.11/OpenNL/include;/tmp/cgal-releases-CGAL-4.11/Operations_on_polyhedra/include;/tmp/cgal-releases-CGAL-4.11/Optimal_transportation_reconstruction_2/include;/tmp/cgal-releases-CGAL-4.11/Optimisation_basic/include;/tmp/cgal-releases-CGAL-4.11/Partition_2/include;/tmp/cgal-releases-CGAL-4.11/Periodic_2_triangulation_2/include;/tmp/cgal-releases-CGAL-4.11/Periodic_3_triangulation_3/include;/tmp/cgal-releases-CGAL-4.11/Point_set_2/include;/tmp/cgal-releases-CGAL-4.11/Point_set_3/include;/tmp/cgal-releases-CGAL-4.11/Point_set_processing_3/include;/tmp/cgal-releases-CGAL-4.11/Point_set_shape_detection_3/include;/tmp/cgal-releases-CGAL-4.11/Poisson_surface_reconstruction_3/include;/tmp/cgal-releases-CGAL-4.11/Polygon/include;/tmp/cgal-releases-CGAL-4.11/Polygon_mesh_processing/include;/tmp/cgal-releases-CGAL-4.11/Polyhedron/include;/tmp/cgal-releases-CGAL-4.11/Polyhedron_IO/include;/tmp/cgal-releases-CGAL-4.11/Polyline_simplification_2/include;/tmp/cgal-releases-CGAL-4.11/Polynomial/include;/tmp/cgal-releases-CGAL-4.11/Polytope_distance_d/include;/tmp/cgal-releases-CGAL-4.11/Principal_component_analysis/include;/tmp/cgal-releases-CGAL-4.11/Principal_component_analysis_LGPL/include;/tmp/cgal-releases-CGAL-4.11/Profiling_tools/include;/tmp/cgal-releases-CGAL-4.11/Property_map/include;/tmp/cgal-releases-CGAL-4.11/QP_solver/include;/tmp/cgal-releases-CGAL-4.11/Random_numbers/include;/tmp/cgal-releases-CGAL-4.11/Ridges_3/include;/tmp/cgal-releases-CGAL-4.11/STL_Extension/include;/tmp/cgal-releases-CGAL-4.11/Scale_space_reconstruction_3/include;/tmp/cgal-releases-CGAL-4.11/Scripts/include;/tmp/cgal-releases-CGAL-4.11/SearchStructures/include;/tmp/cgal-releases-CGAL-4.11/Segment_Delaunay_graph_2/include;/tmp/cgal-releases-CGAL-4.11/Segment_Delaunay_graph_Linf_2/include;/tmp/cgal-releases-CGAL-4.11/Skin_surface_3/include;/tmp/cgal-releases-CGAL-4.11/Snap_rounding_2/include;/tmp/cgal-releases-CGAL-4.11/Solver_interface/include;/tmp/cgal-releases-CGAL-4.11/Spatial_searching/include;/tmp/cgal-releases-CGAL-4.11/Spatial_sorting/include;/tmp/cgal-releases-CGAL-4.11/Straight_skeleton_2/include;/tmp/cgal-releases-CGAL-4.11/Stream_lines_2/include;/tmp/cgal-releases-CGAL-4.11/Stream_support/include;/tmp/cgal-releases-CGAL-4.11/Subdivision_method_3/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh_deformation/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh_parameterization/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh_segmentation/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh_shortest_path/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh_simplification/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesh_skeletonization/include;/tmp/cgal-releases-CGAL-4.11/Surface_mesher/include;/tmp/cgal-releases-CGAL-4.11/Sweep_line_2/include;/tmp/cgal-releases-CGAL-4.11/TDS_2/include;/tmp/cgal-releases-CGAL-4.11/TDS_3/include;/tmp/cgal-releases-CGAL-4.11/Testsuite/include;/tmp/cgal-releases-CGAL-4.11/Three/include;/tmp/cgal-releases-CGAL-4.11/Triangulation/include;/tmp/cgal-releases-CGAL-4.11/Triangulation_2/include;/tmp/cgal-releases-CGAL-4.11/Triangulation_3/include;/tmp/cgal-releases-CGAL-4.11/Union_find/include;/tmp/cgal-releases-CGAL-4.11/Visibility_2/include;/tmp/cgal-releases-CGAL-4.11/Voronoi_diagram_2/include;/tmp/cgal-releases-CGAL-4.11/wininst/include" )
set(CGAL_MODULES_DIR       "/tmp/cgal-releases-CGAL-4.11/Installation/cmake/modules" )
set(CGAL_LIBRARIES_DIR     "/tmp/cgal-releases-CGAL-4.11/buildR/lib" )

# If CGAL_ImageIO is built, tell if it was linked with Zlib.
set(CGAL_ImageIO_USE_ZLIB                 "ON" )

set(CGAL_VERSION "${CGAL_MAJOR_VERSION}.${CGAL_MINOR_VERSION}.${CGAL_BUGFIX_VERSION}")

set(CGAL_USE_FILE "${CGAL_MODULES_DIR}/UseCGAL.cmake" )

if ( CGAL_FIND_REQUIRED )
  set( CHECK_CGAL_COMPONENT_MSG_ON_ERROR TRUE        )
  set( CHECK_CGAL_COMPONENT_ERROR_TYPE   FATAL_ERROR )
  set( CHECK_CGAL_COMPONENT_ERROR_TITLE  "ERROR:"    )
else()
  if ( NOT CGAL_FIND_QUIETLY )
    set( CHECK_CGAL_COMPONENT_MSG_ON_ERROR TRUE      )
    set( CHECK_CGAL_COMPONENT_ERROR_TYPE   STATUS    )
    set( CHECK_CGAL_COMPONENT_ERROR_TITLE "NOTICE:" )
  else()
    set( CHECK_CGAL_COMPONENT_MSG_ON_ERROR FALSE )
  endif()
endif()

set(CGAL_CONFIGURED_LIBRARIES "CGAL_Core;CGAL_ImageIO;CGAL_Qt5")

macro(check_cgal_component COMPONENT)

  set( CGAL_LIB ${COMPONENT} )
  #message("LIB: ${CGAL_LIB}")

  if ( "${CGAL_LIB}" STREQUAL "CGAL" )
    set( CGAL_FOUND TRUE )
    set( CHECK_CGAL_ERROR_TAIL "" )
  else( "${CGAL_LIB}" STREQUAL "CGAL" )
    if ( WITH_${CGAL_LIB} )
      if(CGAL_HEADER_ONLY OR TARGET CGAL::${CGAL_LIB})
        if ("${CGAL_LIB}" STREQUAL "CGAL_Qt5")
          find_package(Qt5 QUIET COMPONENTS OpenGL Svg)
          find_package(OpenGL QUIET)

          set(CGAL_Qt5_MISSING_DEPS "")
          if(NOT Qt5OpenGL_FOUND)
            set(CGAL_Qt5_MISSING_DEPS "Qt5OpenGL")
          endif()
          if(NOT Qt5Svg_FOUND)
            set(CGAL_Qt5_MISSING_DEPS "${CGAL_Qt5_MISSING_DEPS} Qt5Svg")
          endif()
          if(NOT Qt5_FOUND)
            set(CGAL_Qt5_MISSING_DEPS "${CGAL_Qt5_MISSING_DEPS} Qt5")
          endif()
          if(NOT OPENGL_FOUND)
            set(CGAL_Qt5_MISSING_DEPS "${CGAL_Qt5_MISSING_DEPS} OpenGL")
          endif()

          if(CGAL_Qt5_MISSING_DEPS)
            set( ${CGAL_LIB}_FOUND FALSE )
            message(STATUS "libCGAL_Qt5 is missing the dependencies: ${CGAL_Qt5_MISSING_DEPS} cannot be configured.")
          else()
            set( ${CGAL_LIB}_FOUND TRUE )
            if (CGAL_HEADER_ONLY)
              include("${CGAL_MODULES_DIR}/Use_CGAL_Qt5_headers.cmake")
            endif()
          endif()
        else("${CGAL_LIB}" STREQUAL "CGAL_Qt5")
          # Librairies that have no dependencies
          set( ${CGAL_LIB}_FOUND TRUE )
        endif("${CGAL_LIB}" STREQUAL "CGAL_Qt5")
      else(CGAL_HEADER_ONLY OR TARGET CGAL::${CGAL_LIB})
        set( ${CGAL_LIB}_FOUND FALSE )
        set( CHECK_${CGAL_LIB}_ERROR_TAIL " CGAL was configured with WITH_${CGAL_LIB}=ON, but one of the dependencies of ${CGAL_LIB} was not configured properly." )
      endif(CGAL_HEADER_ONLY OR TARGET CGAL::${CGAL_LIB})
    else( WITH_${CGAL_LIB} )
      set( ${CGAL_LIB}_FOUND FALSE )
      set( CHECK_${CGAL_LIB}_ERROR_TAIL " Please configure CGAL using WITH_${CGAL_LIB}=ON." )
    endif( WITH_${CGAL_LIB} )
  endif( "${CGAL_LIB}" STREQUAL "CGAL" )

  if ( NOT ${CGAL_LIB}_FOUND AND CHECK_CGAL_COMPONENT_MSG_ON_ERROR )
    message( ${CHECK_CGAL_COMPONENT_ERROR_TYPE} "${CHECK_CGAL_COMPONENT_ERROR_TITLE} The ${CGAL_LIB} library was not configured.${CHECK_${CGAL_LIB}_ERROR_TAIL}" )
  endif()

endmacro()

check_cgal_component("CGAL")

foreach( CGAL_COMPONENT ${CGAL_FIND_COMPONENTS} )
  list (FIND CGAL_CONFIGURED_LIBRARIES "CGAL_${CGAL_COMPONENT}" POSITION)
  if ("${POSITION}" STRGREATER "-1") # means: CGAL_COMPONENT is contained in list
    check_cgal_component("CGAL_${CGAL_COMPONENT}")
# TODO EBEB do something for supporting lib in check_component?
  endif()
endforeach()

# Starting with cmake 2.6.3, CGAL_FIND_COMPONENTS is cleared out when find_package returns.
# But we need it within UseCGAL.cmake, so we save it aside into another variable
set( CGAL_REQUESTED_COMPONENTS ${CGAL_FIND_COMPONENTS} )

# for preconfigured libs
set(CGAL_ENABLE_PRECONFIG "ON")
set(CGAL_SUPPORTING_3RD_PARTY_LIBRARIES "GMP;GMPXX;MPFR;ZLIB;OpenGL;LEDA;MPFI;RS;RS3;OpenNL;Eigen3;BLAS;LAPACK;QGLViewer;ESBTL;Coin3D;NTL;IPE")
set(CGAL_ESSENTIAL_3RD_PARTY_LIBRARIES "GMP;MPFR")

set(CGAL_DISABLE_GMP "")
set(CGAL_EXT_LIB_Qt5_PREFIX "QT")
set(CGAL_EXT_LIB_Eigen3_PREFIX "EIGEN3")
set(CGAL_EXT_LIB_QGLViewer_PREFIX "QGLVIEWER")
set(CGAL_EXT_LIB_Coin3D_PREFIX "COIN3D")

if (NOT CGAL_IGNORE_PRECONFIGURED_GMP)
  set( GMP_FOUND           "TRUE" )
  set( GMP_USE_FILE        "" )
  set( GMP_INCLUDE_DIR     "/usr/include/x86_64-linux-gnu" )
  set( GMP_LIBRARIES       "/usr/lib/x86_64-linux-gnu/libgmp.so" )
  set( GMP_DEFINITIONS     "" )
endif()

if (NOT CGAL_IGNORE_PRECONFIGURED_MPFR)
  set( MPFR_FOUND           "TRUE" )
  set( MPFR_USE_FILE        "" )
  set( MPFR_INCLUDE_DIR     "/usr/include" )
  set( MPFR_LIBRARIES       "/usr/lib/x86_64-linux-gnu/libmpfr.so" )
  set( MPFR_DEFINITIONS     "" )
endif()

